/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdint.h>
#include <ram_pwrdn.h>
#include <zephyr/kernel.h>
#include <zephyr/pm/device.h>
#include <zephyr/device.h>
#include <zephyr/sys/poweroff.h>

#define SYSTEM_ON_IDLE_DURATION_MS  (1000 * 10)

static struct device const *uart_dev = DEVICE_DT_GET(DT_CHOSEN(zephyr_console));

enum ram_config {
	RAM_ALL_ON,
	RAM_128K,
	RAM_64K
};

void current_meas_scenario(enum ram_config config) 
{
	int err; 

	(void) pm_device_action_run(uart_dev, PM_DEVICE_ACTION_RESUME);

	switch (config) {
		case RAM_ALL_ON:
			printk("Enter System ON mode with all RAM enabled\n");
			break;

		case RAM_128K:
			printk("Enter System ON mode with 128K of RAM enabled\n");
			power_down_ram(0x20020000, 0x20040000);
			break;

		case RAM_64K:
			printk("Enter System ON mode with 64K of RAM enabled\n");
			power_down_ram(0x20010000, 0x20040000);
			break;
	}

	k_msleep(100);
	/* Disable UART to reduce System ON idle current */
	err = pm_device_action_run(uart_dev, PM_DEVICE_ACTION_SUSPEND);
	if (err) {
		printk("UART suspend failed. (err %d)\n", err);
		return;
	}

	/* Enter System ON idle */
	k_msleep(SYSTEM_ON_IDLE_DURATION_MS);

	power_up_ram(0x20000000, 0x20040000);
}


int main(void)
{
	
	printk("Hello World! %s\n", CONFIG_BOARD_TARGET);

	current_meas_scenario(RAM_ALL_ON);
	current_meas_scenario(RAM_128K);
	current_meas_scenario(RAM_64K);

	sys_poweroff();

	return 0;
}
